@@ -11,7 +11,11 @@ Arguments:
   <you>  : Your name [default: Anon]
   <me>  : My name [default: Casper]
 """
-import sys, argopt, shtab  # NOQA
+import sys  # NOQA
+
+import argopt
+
+import shtab
 
 parser = argopt.argopt(__doc__)
 shtab.add_argument_to(parser, ["-s", "--print-completion"])  # magic!
