@@ -16,31 +16,16 @@ from argparse import (
 )
 from functools import total_ordering
 
-
-def get_version_dist(name=__name__):
-    from pkg_resources import DistributionNotFound, get_distribution
-
-    try:
-        return get_distribution(name).version
-    except DistributionNotFound:
-        return "UNKNOWN"
-
-
+# version detector. Precedence: installed dist, git, 'UNKNOWN'
 try:
-    from setuptools_scm import get_version
+    from ._dist_ver import __version__
 except ImportError:
-    from os import path
-
-    ROOT = path.abspath(path.dirname(path.dirname(__file__)))
-    if path.exists(path.join(ROOT, ".git")):
-        __version__ = "UNKNOWN - please install setuptools_scm"
-    else:
-        __version__ = get_version_dist()
-else:
     try:
+        from setuptools_scm import get_version
+
         __version__ = get_version(root="..", relative_to=__file__)
-    except LookupError:
-        __version__ = get_version_dist()
+    except (ImportError, LookupError):
+        __version__ = "UNKNOWN"
 __all__ = [
     "complete",
     "add_argument_to",
@@ -153,8 +138,8 @@ def replace_format(string, **fmt):
 
 
 def wordify(string):
-    """Replace hyphens (-) and spaces ( ) with underscores (_)"""
-    return string.replace("-", "_").replace(" ", "_")
+    """Replace non-word chars [-. ] with underscores [_]"""
+    return string.replace("-", "_").replace(".", " ").replace(" ", "_")
 
 
 def get_bash_commands(root_parser, root_prefix, choice_functions=None):
@@ -217,8 +202,7 @@ def get_bash_commands(root_parser, root_prefix, choice_functions=None):
             if hasattr(sub, "complete"):
                 print(
                     u"{}_COMPGEN={}".format(
-                        prefix,
-                        complete2pattern(sub.complete, "bash", choice_type2fn),
+                        prefix, complete2pattern(sub.complete, "bash", choice_type2fn),
                     ),
                     file=fd,
                 )
@@ -226,15 +210,9 @@ def get_bash_commands(root_parser, root_prefix, choice_functions=None):
                 log.debug("choices:{}:{}".format(prefix, sorted(sub.choices)))
                 for cmd in sorted(sub.choices):
                     if isinstance(cmd, Choice):
-                        log.debug(
-                            "Choice.{}:{}:{}".format(
-                                cmd.type, prefix, sub.dest
-                            )
-                        )
+                        log.debug("Choice.{}:{}:{}".format(cmd.type, prefix, sub.dest))
                         print(
-                            u"{}_COMPGEN={}".format(
-                                prefix, choice_type2fn[cmd.type]
-                            ),
+                            u"{}_COMPGEN={}".format(prefix, choice_type2fn[cmd.type]),
                             file=fd,
                         )
                     elif isinstance(sub.choices, dict):
@@ -303,6 +281,11 @@ _shtab_replace_hyphen() {
   echo $1 | sed 's/-/_/g'
 }
 
+# $1=COMP_WORDS[1]
+_shtab_replace_nonword() {
+  echo "${1//[^[:word:]]/_}"
+}
+
 # $1=COMP_WORDS[1]
 {root_prefix}_compgen_root_() {
   local args_gen="{root_prefix}_COMPGEN"
@@ -316,7 +299,7 @@ _shtab_replace_hyphen() {
 
 # $1=COMP_WORDS[1]
 {root_prefix}_compgen_command_() {
-  local flags_list="{root_prefix}_$(_shtab_replace_hyphen $1)"
+  local flags_list="{root_prefix}_$(_shtab_replace_nonword $1)"
   local args_gen="${flags_list}_COMPGEN"
   COMPREPLY=( $(compgen -W "${!flags_list}" -- "$word"; \
 [ -n "${!args_gen}" ] && ${!args_gen} "$word") )
@@ -325,8 +308,7 @@ _shtab_replace_hyphen() {
 # $1=COMP_WORDS[1]
 # $2=COMP_WORDS[2]
 {root_prefix}_compgen_subcommand_() {
-  local flags_list="{root_prefix}_$(\
-_shtab_replace_hyphen $1)_$(_shtab_replace_hyphen $2)"
+  local flags_list="{root_prefix}_$(_shtab_replace_nonword "${1}_${2}")"
   local args_gen="${flags_list}_COMPGEN"
   [ -n "${!args_gen}" ] && local opts_more="$(${!args_gen} "$word")"
   local opts="${!flags_list}"
@@ -476,9 +458,7 @@ def complete_zsh(parser, root_prefix=None, preamble="", choice_functions=None):
                     [],
                 )
                 if subsubs:
-                    arguments.append(
-                        '"1:Sub command:({})"'.format(" ".join(subsubs))
-                    )
+                    arguments.append('"1:Sub command:({})"'.format(" ".join(subsubs)))
 
                 # positionals
                 arguments.extend(
@@ -489,9 +469,7 @@ def complete_zsh(parser, root_prefix=None, preamble="", choice_functions=None):
                 )
 
                 subcommands[cmd] = {
-                    "help": (subparser.description or "")
-                    .strip()
-                    .split("\n")[0],
+                    "help": (subparser.description or "").strip().split("\n")[0],
                     "arguments": arguments,
                 }
                 log.debug("subcommands:%s:%s", cmd, subcommands[cmd])
@@ -598,34 +576,40 @@ def complete(
     )
 
 
-class PrintCompletionAction(Action):
-    def __call__(self, parser, namespace, values, option_string=None):
-        print(complete(parser, values))
-        parser.exit(0)
+def completion_action(parent=None):
+    class PrintCompletionAction(Action):
+        def __call__(self, parser, namespace, values, option_string=None):
+            print(complete(parent or parser, values))
+            parser.exit(0)
+
+    return PrintCompletionAction
 
 
 def add_argument_to(
     parser,
     option_string="--print-completion",
     help="print shell completion script",
+    parent=None,
 ):
     """
     parser  : argparse.ArgumentParser
     option_string  : str or list[str], iff positional (no `-` prefix) then
       `parser` is assumed to actually be a subparser (subcommand mode)
     help  : str
+    parent  : argparse.ArgumentParser, required in subcommand mode
     """
     if isinstance(
         option_string, str if sys.version_info[0] > 2 else basestring  # NOQA
     ):
         option_string = [option_string]
-    kwargs = dict(
-        choices=SUPPORTED_SHELLS,
-        default=None,
-        help=help,
-        action=PrintCompletionAction,
-    )
+    kwargs = {
+        "choices": SUPPORTED_SHELLS,
+        "default": None,
+        "help": help,
+        "action": completion_action(parent),
+    }
     if option_string[0][0] != "-":  # subparser mode
         kwargs.update(default=SUPPORTED_SHELLS[0], nargs="?")
+        assert parent is not None, "subcommand mode: parent required"
     parser.add_argument(*option_string, **kwargs)
     return parser
