@@ -38,9 +38,7 @@ class Bash(object):
             failure_message, test, stdout or "", stderr or ""
         )
 
-    def compgen(
-        self, compgen_cmd, word, expected_completions, failure_message=""
-    ):
+    def compgen(self, compgen_cmd, word, expected_completions, failure_message=""):
         self.test(
             '"$(echo $(compgen {} -- "{}"))" = "{}"'.format(
                 compgen_cmd, word, expected_completions
@@ -91,6 +89,27 @@ def test_prog_override(shell, caplog, capsys):
     assert not caplog.record_tuples
 
 
+@fix_shell
+def test_prog_scripts(shell, caplog, capsys):
+    with caplog.at_level(logging.INFO):
+        main(["-s", shell, "--prog", "script.py", "shtab.main.get_main_parser"])
+
+    captured = capsys.readouterr()
+    assert not captured.err
+    script_py = [i.strip() for i in captured.out.splitlines() if "script.py" in i]
+    if shell == "bash":
+        assert script_py == ["complete -o nospace -F _shtab_shtab script.py"]
+    elif shell == "zsh":
+        assert script_py == [
+            "#compdef script.py",
+            "_describe 'script.py commands' _commands",
+        ]
+    else:
+        raise NotImplementedError(shell)
+
+    assert not caplog.record_tuples
+
+
 @fix_shell
 def test_prefix_override(shell, caplog, capsys):
     with caplog.at_level(logging.INFO):
@@ -138,9 +157,7 @@ def test_positional_choices(shell, caplog):
 def test_custom_complete(shell, caplog):
     parser = ArgumentParser(prog="test")
     parser.add_argument("posA").complete = {"bash": "_shtab_test_some_func"}
-    preamble = {
-        "bash": "_shtab_test_some_func() { compgen -W 'one two' -- $1 ;}"
-    }
+    preamble = {"bash": "_shtab_test_some_func() { compgen -W 'one two' -- $1 ;}"}
     with caplog.at_level(logging.INFO):
         completion = shtab.complete(parser, shell=shell, preamble=preamble)
     print(completion)
@@ -158,9 +175,7 @@ def test_subparser_custom_complete(shell, caplog):
     subparsers = parser.add_subparsers()
     sub = subparsers.add_parser("sub")
     sub.add_argument("posA").complete = {"bash": "_shtab_test_some_func"}
-    preamble = {
-        "bash": "_shtab_test_some_func() { compgen -W 'one two' -- $1 ;}"
-    }
+    preamble = {"bash": "_shtab_test_some_func() { compgen -W 'one two' -- $1 ;}"}
     with caplog.at_level(logging.INFO):
         completion = shtab.complete(parser, shell=shell, preamble=preamble)
     print(completion)
@@ -189,17 +204,27 @@ def test_add_argument_to_optional(shell, caplog):
 
 
 @fix_shell
-def test_add_argument_to_positional(shell, caplog):
+def test_add_argument_to_positional(shell, caplog, capsys):
     parser = ArgumentParser(prog="test")
     subparsers = parser.add_subparsers()
     sub = subparsers.add_parser("completion")
-    shtab.add_argument_to(sub, "shell")
+    shtab.add_argument_to(sub, "shell", parent=parser)
+    from argparse import Namespace
+
     with caplog.at_level(logging.INFO):
-        completion = shtab.complete(parser, shell=shell)
+        completion_manual = shtab.complete(parser, shell=shell)
+        with pytest.raises(SystemExit) as exc:
+            sub._actions[-1](sub, Namespace(), shell)
+            assert exc.type == SystemExit
+            assert exc.vaue.code == 0
+    completion, err = capsys.readouterr()
     print(completion)
+    assert completion_manual == completion.rstrip()
+    assert not err
 
     if shell == "bash":
         shell = Bash(completion)
+        shell.compgen('-W "$_shtab_test_commands_"', "c", "completion")
         shell.compgen('-W "$_shtab_test_completion"', "ba", "bash")
         shell.compgen('-W "$_shtab_test_completion"', "z", "zsh")
 
